#import "ImageEncodingManager.h"
#import <SDWebImage/SDWebImage.h>
#import <SDWebImageAVIFCoder/SDWebImageAVIFCoder.h>

@interface ImageEncodingManager ()
@property(nonatomic, strong) dispatch_queue_t encodingQueue;
@end

@implementation ImageEncodingManager

+ (instancetype)sharedManager {
  static ImageEncodingManager *manager = nil;
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    manager = [[ImageEncodingManager alloc] init];
  });
  return manager;
}

- (instancetype)init {
  self = [super init];
  if (self) {
    dispatch_queue_attr_t attr = dispatch_queue_attr_make_with_qos_class(
        DISPATCH_QUEUE_SERIAL, QOS_CLASS_USER_INITIATED, 0);
    self.encodingQueue =
        dispatch_queue_create("com.aliyun.encodingQueue", attr);
  }
  return self;
}

- (void)encodeImages:(NSArray<ImageData *> *)images
          completion:(void (^)(BOOL success,
                               CFTimeInterval totalTime))completion
     progressHandler:(void (^)(NSString *status, NSInteger current,
                               NSInteger total))progressHandler {

  __block NSInteger currentIndex = 0;
  NSString *documentsPath = [NSSearchPathForDirectoriesInDomains(
      NSDocumentDirectory, NSUserDomainMask, YES) firstObject];
  NSString *encodedImagesPath =
      [documentsPath stringByAppendingPathComponent:@"encoded-images"];

  if (![[NSFileManager defaultManager] fileExistsAtPath:encodedImagesPath]) {
    [[NSFileManager defaultManager] createDirectoryAtPath:encodedImagesPath
                              withIntermediateDirectories:YES
                                               attributes:nil
                                                    error:nil];
    }

    __block CFTimeInterval totalEncodeTime = 0;

    void (^encodeNextImage)(void);
    __block void (^weakEncodeNextImage)(void);

    encodeNextImage = ^{
      if (currentIndex >= images.count) {
        dispatch_async(dispatch_get_main_queue(), ^{
          if (completion) {
            completion(YES, totalEncodeTime);
          }
        });
        return;
      }

      ImageData *imageData = images[currentIndex];
      NSURL *imageURL = [NSURL URLWithString:imageData.url];

      dispatch_async(dispatch_get_main_queue(), ^{
        if (progressHandler) {
          progressHandler(@"正在编码...", currentIndex + 1, images.count);
        }
      });

      [[SDWebImageManager sharedManager]
          loadImageWithURL:imageURL
                   options:0
                  progress:nil
                 completed:^(UIImage *_Nullable image, NSData *_Nullable data,
                             NSError *_Nullable error,
                             SDImageCacheType cacheType, BOOL finished,
                             NSURL *_Nullable imageURL) {
                   [self encodeImage:image
                               imageData:imageData
                       encodedImagesPath:encodedImagesPath
                         totalEncodeTime:&totalEncodeTime];
                   currentIndex++;
                   dispatch_async(dispatch_get_main_queue(), ^{
                     weakEncodeNextImage();
                   });
                 }];
    };

    weakEncodeNextImage = encodeNextImage;
    encodeNextImage();
}

- (void)encodeImage:(UIImage *)image
            imageData:(ImageData *)imageData
    encodedImagesPath:(NSString *)encodedImagesPath
      totalEncodeTime:(CFTimeInterval *)totalEncodeTime {

  if (image) {
    dispatch_async(self.encodingQueue, ^{
      CFTimeInterval startEncodingTime = CACurrentMediaTime();
      id<SDImageCoder> coder = [SDImageCodersManager sharedManager];
      NSData *encodedData = [coder encodedDataWithImage:image
                                                 format:SDImageFormatAVIF
                                                options:nil];
      CFTimeInterval endEncodingTime = CACurrentMediaTime();
      CFTimeInterval encodeTime = endEncodingTime - startEncodingTime;
      *totalEncodeTime += encodeTime;
      imageData.encodingTime = encodeTime;

      if (encodedData) {
        NSString *filePath = [encodedImagesPath
            stringByAppendingPathComponent:[NSString
                                               stringWithFormat:@"%@.avif",
                                                                imageData
                                                                    .object]];
        NSError *writeError = nil;
        BOOL success = [encodedData writeToFile:filePath
                                        options:NSDataWritingAtomic
                                          error:&writeError];

        if (!success) {
          NSLog(@"Failed to write encoded data for image %@: %@",
                imageData.object, writeError);
          imageData.encodingTime = -1;
        }
      } else {
        NSLog(@"Failed to encode image %@", imageData.object);
        imageData.encodingTime = -1;
      }
    });
  } else {
    imageData.encodingTime = -1;
  }
}

@end
