//
// /*
// * Copyright 2009-2021 Alibaba Cloud All rights reserved.
// *
// * Licensed under the Apache License, Version 2.0 (the "License");
// * you may not use this file except in compliance with the License.
// * You may obtain a copy of the License at
// *
// *      http://www.apache.org/licenses/LICENSE-2.0
// *
// * Unless required by applicable law or agreed to in writing, software
// * distributed under the License is distributed on an "AS IS" BASIS,
// * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// * See the License for the specific language governing permissions and
// * limitations under the License.
// *

#import "ViewController.h"
#import "CSVParser.h"
#import "ImageData.h"
#import "ImageDownloadManager.h"
#import "ImageEncodingManager.h"
#import "ImageTableViewCell.h"
#import <MBProgressHUD/MBProgressHUD.h>
#import <Network/Network.h>
#import <SDWebImage/SDWebImage.h>
#import <SDWebImageAVIFCoder/SDWebImageAVIFCoder.h>

typedef NS_ENUM(NSInteger, ImageDisplayStage) {
    ImageDisplayStageInit,
    ImageDisplayStageLoading,
    ImageDisplayStageDownloaded,
    ImageDisplayStageEncoded
};

@interface ViewController () <UITableViewDataSource, UITableViewDelegate>
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (nonatomic, copy) NSArray<ImageData *> *testImages;
@property(nonatomic, assign) ImageDisplayStage stage;
@property(nonatomic, strong) nw_path_monitor_t networkMonitor;
@property(nonatomic, assign) BOOL isNetworkAuthorized;
@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.tableView registerNib:[UINib nibWithNibName:@"ImageTableViewCell" bundle:nil] forCellReuseIdentifier:@"ImageTableViewCell"];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.stage = ImageDisplayStageInit;

    self.isNetworkAuthorized = NO;
    [self setupNetworkMonitoring];
}

- (void)setupNetworkMonitoring {
  self.networkMonitor = nw_path_monitor_create();
  nw_path_monitor_set_queue(self.networkMonitor, dispatch_get_main_queue());

  __weak typeof(self) weakSelf = self;
  nw_path_monitor_set_update_handler(
      self.networkMonitor, ^(nw_path_t _Nonnull path) {
        if (nw_path_get_status(path) == nw_path_status_satisfied) {
          weakSelf.isNetworkAuthorized = YES;
        } else if (nw_path_get_status(path) == nw_path_status_unsatisfied) {
          weakSelf.isNetworkAuthorized = NO;
        }
      });

  nw_path_monitor_start(self.networkMonitor);
}

- (IBAction)startToDecode:(id)sender {
  if (!self.isNetworkAuthorized) {
    UIAlertController *alert = [UIAlertController
        alertControllerWithTitle:@"网络访问受限"
                         message:@"请在设置中允许应用访问网络"
                  preferredStyle:UIAlertControllerStyleAlert];

    UIAlertAction *settingsAction = [UIAlertAction
        actionWithTitle:@"去设置"
                  style:UIAlertActionStyleDefault
                handler:^(UIAlertAction *_Nonnull action) {
                  NSURL *settingsURL =
                      [NSURL URLWithString:UIApplicationOpenSettingsURLString];
                  if ([[UIApplication sharedApplication]
                          canOpenURL:settingsURL]) {
                    [[UIApplication sharedApplication] openURL:settingsURL
                                                       options:@{}
                                             completionHandler:nil];
                  }
                }];

    UIAlertAction *cancelAction =
        [UIAlertAction actionWithTitle:@"取消"
                                 style:UIAlertActionStyleCancel
                               handler:nil];

    [alert addAction:settingsAction];
    [alert addAction:cancelAction];
    [self presentViewController:alert animated:YES completion:nil];
    return;
  }

    if (self.stage == ImageDisplayStageInit) {
        [self loadCSV];
    }
}

- (void)loadCSV {
    self.stage = ImageDisplayStageLoading;
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    hud.label.text = @"加载测试数据...";
    CSVParser *parser = [[CSVParser alloc] init];
    self.testImages = [parser parseCSVFile:[[NSBundle mainBundle] pathForResource:@"test_urls" ofType:@"csv"]];
    
    hud.label.text = @"下载原图...";
    [self downloadImages];
}

- (void)downloadImages {
  [[ImageDownloadManager sharedManager] downloadImagesWithArray:self.testImages
      completion:^(BOOL success) {
        self.stage = ImageDisplayStageDownloaded;
        MBProgressHUD *hud = [MBProgressHUD HUDForView:self.view];
        hud.label.text = @"编码图片中...";
        [self encodeImages];
      }
      progressHandler:^(NSString *status) {
        MBProgressHUD *hud = [MBProgressHUD HUDForView:self.view];
        hud.label.text = status;
      }];
}

- (void)encodeImages {
  [[ImageEncodingManager sharedManager] encodeImages:self.testImages
      completion:^(BOOL success, CFTimeInterval totalTime) {
        self.stage = ImageDisplayStageEncoded;
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        [self.tableView reloadData];
        NSLog(@"Total encode time: %f seconds, average time:%f ms", totalTime,
              totalTime / self.testImages.count * 1000);
      }
      progressHandler:^(NSString *status, NSInteger current, NSInteger total) {
        MBProgressHUD *hud = [MBProgressHUD HUDForView:self.view];
        hud.label.text = [NSString
            stringWithFormat:@"%@ %ld/%ld", status, (long)current, (long)total];
      }];
}

#pragma mark - UITableViewDataSource

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 100;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.testImages.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    ImageTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"ImageTableViewCell" forIndexPath:indexPath];
    if (self.stage == ImageDisplayStageEncoded) {
        ImageData *imageData = self.testImages[indexPath.row];
        [cell.previousImageView
            sd_setImageWithURL:[NSURL URLWithString:imageData.url]];

        if (imageData.encodingTime >= 0) {
          NSString *documentsPath = [NSSearchPathForDirectoriesInDomains(
              NSDocumentDirectory, NSUserDomainMask, YES) firstObject];
          NSString *encodedImagePath = [documentsPath
              stringByAppendingPathComponent:
                  [NSString stringWithFormat:@"encoded-images/%@.avif",
                                             imageData.object]];
          [cell.currentImageView
              sd_setImageWithURL:[NSURL fileURLWithPath:encodedImagePath]];
          cell.encodingDescLabel.text =
              [NSString stringWithFormat:@"编码后,编码耗时: %.3f ms",
                                         imageData.encodingTime * 1000];
        } else {
          cell.currentImageView.image = nil;
          cell.encodingDescLabel.text = @"编码失败";
        }

        cell.previousImageDescLabel.text = [NSString stringWithFormat:@"原图: %ldx%ld", imageData.width, imageData.height];
    }
    return cell;
}

- (void)dealloc {
  if (self.networkMonitor) {
    nw_path_monitor_cancel(self.networkMonitor);
  }
}

@end
